# Meta information for the project.
__version__ = "0.1.0"
__author__ = "Nyakku Shigure"
__year__ = "2022"
__project_info__ = {
    "name": __name__,
    "version": __version__,
    "copyright": f"{__year__}, {__author__}",
    "author": __author__,
}

# The exported modules.
from ._abc import Formatter

__all__ = [
    "Formatter",
]
